/*
 * Copyright (C) 2020 Rerrah
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

#pragma once

#include <cstdint>
#include <vector>
#include "binary_container.hpp"

namespace io
{
class WavContainer
{
public:
	explicit WavContainer(size_t defCapacity = 0, uint32_t rate = 44100, uint16_t nCh = 2, uint16_t getBitSize = 16);
	explicit WavContainer(const BinaryContainer& bc);

	void setChannelCount(uint16_t n) noexcept;
	uint16_t getChannelCount() const noexcept;
	void setBitSize(uint16_t size) noexcept;
	uint16_t getBitSize() const noexcept;
	void setSampleRate(uint32_t rate) noexcept;
	uint32_t getSampleRate() const noexcept;

	size_t getSampleCount() const;

	void appendSample(const int16_t* sample, size_t nSamples);
	void appendSample(const std::vector<int16_t>& sample);
	void appendSample(const BinaryContainer& sample);
	BinaryContainer getSample() const noexcept;

	BinaryContainer createWavBinary();

	//		static WavContainer* resample(const WavContainer* src, uint32_t rate);
	//		static WavContainer* mono(const WavContainer* src);

private:
	uint16_t nCh_, bitSize_;
	uint32_t rate_;
	BinaryContainer buf_;
};
}
